@@ -10,7 +10,7 @@ from .backend import *
 import h5py
 
 class Meshoid(object):
-    def __init__(self, pos, m=None, hsml=None, des_ngb=None, boxsize=None, verbose=False, particle_mask=None, n_jobs=1):
+    def __init__(self, pos, m=None, hsml=None, des_ngb=None, boxsize=-1, verbose=False, particle_mask=None, n_jobs=1):
         """
         Creates a Meshoid object instance given at least the positions of the particles.
         
@@ -116,7 +116,7 @@ class Meshoid(object):
         if self.verbose: print("Finding nearest neighbours...")
                 
         self.tree = cKDTree(self.pos, boxsize=self.boxsize)
-        self.ngbdist, self.ngb = self.tree.query(self.pos[self.particle_mask], self.des_ngb, n_jobs=self.n_jobs)
+        self.ngbdist, self.ngb = self.tree.query(self.pos[self.particle_mask], self.des_ngb, workers=self.n_jobs)
                 
         if self.verbose: print("Neighbours found!")
 
@@ -322,6 +322,44 @@ class Meshoid(object):
         else:
             return np.einsum('ij,ij...->i...', self.sliceweights, f[ngb]).reshape((res,res))
 
+    def InterpToGrid(self, f, weights=None, size=None, center=None, res=128, method='kernel'):
+        """
+        Interpolates the quantity f defined on the meshoid to a Cartesian grid
+
+        """
+        if center is None: center = self.center
+        if size is None: size = self.L
+        if weights is None: weights = self.m
+
+        x = np.linspace(-size/2,size/2,res+1)
+        x = (x[1:] + x[:-1])/2
+        X, Y, Z = np.meshgrid(x,x,x,indexing='ij')
+        gridcoords = np.c_[X.flatten(),Y.flatten(),Z.flatten()] + center
+        if method=='nearest':
+            ngbdist, ngb = self.tree.query(gridcoords, workers=self.n_jobs)
+            f_interp = f[ngb].reshape((res,res,res))
+        elif method=='kernel':
+            h = np.clip(self.hsml, 2 * size/(res-1), 1e100)
+            f_interp = WeightedGridInterp3D(f, weights, self.pos, h, center, size, res=res,box_size=self.boxsize)
+                   
+        return f_interp
+
+    def DepositToGrid(self, f, weights=None, size=None, center=None, res=128):
+        """
+        Deposits a conserved quantity (e.g. mass, momentum, energy) to a 3D grid and returns the density of that quantity that grid
+        """
+        if center is None: center = self.center
+        if size is None: size = self.L
+        if weights is None: weights = self.m
+
+        h = np.clip(self.hsml, 2 * size/(res-1), 1e100)
+        f_grid = GridDensity(f, self.pos, h, center, size, res=res,box_size=self.boxsize)
+                   
+        return f_grid
+
+        
+        
+
     def SurfaceDensity(self, f=None, size=None, plane='z', center=None, res=128, smooth_fac=1.):
         """
         Computes the surface density of a quantity f defined on the meshoid on a grid of sightlines. e.g. if f is the particle masses, you will get mass surface density.
@@ -329,7 +367,7 @@ class Meshoid(object):
         Keyword arguments:
         f - the quantity you want the surface density of (default: particle mass)
         size - the side length of the window of sightlines (default: None, will use the meshoid's predefined side length')
-        plane - the direction you want to project along, of x, y, or z (default: 'z'')
+        plane - the direction you want to project along, of x, y, or z (default: 'z')
         center - (2,) or (3,) array containing the coordaintes of the center of the grid (default: None, will use the meshoid's predefined center)
         res - the resolution of the grid of sightlines (default: 128)
         smooth_fac - smoothing lengths are increased by this factor (default: 1.)
@@ -340,7 +378,7 @@ class Meshoid(object):
         if f is None: f = self.m
         if center is None: center = self.center
         if size is None: size = self.L
-        return GridSurfaceDensity(f, self.pos, np.clip(smooth_fac*self.hsml, 2*size/res,1e100), center, size, res, self.boxsize)
+        return GridSurfaceDensity(f, self.pos, np.clip(smooth_fac*self.hsml, 2*size/res,1e100), center, size, res, self.boxsize, parallel=(False if self.n_jobs == 1 else True))
 
     def ProjectedAverage(self, f, size=None, plane='z', center=None, res=128, smooth_fac=1.):
         """
